<?php

/*
 *	Article System
 *	Online journal management tool
 written in PHP/MySQL.
 *	This code is available at http://sourceforge.net/projects/artsys
 *
 *	Copyright (C) 2002 Jan Hnatek
 *	Distributed under the terms of the GNU General Public License
 *
 *	Date:	  07/05/2002
 *	Version:  0.1-1 (pre-alpha)      	
 */

###############################################################################

/**	
 *	Abstract class for an entity in DB.<br>
 *	Used to simplify <code>SELECT</code> SQL queries and rendering of the results.
 *	
 *	@author	Jan Hnatek
 */
class db_entity {

	// VARIABLES
	//
	var $name;
	var $error;
	var $result = array();

	// CONSTRUCTOR
	//
	/**	Contructor of db_entity class.<br>
	 *	Needs to set the name of the DB entity.
	 *	@param	$iname	string	Name of the table in DB describing the entity.
	 *	@access	public
	 */
	function db_entity ($iname) {
		$this->name = $iname;
	}

	// METHODS
	//
	/**	Function executing a DB SELECT query.
	 *	@param	$what	string	The text between 'SELECT' and 'FROM'. Column names or "*".
	 *	@param	$where	string	Optional WHERE/ORDER clause.
	 *	@access	public
	 *	@return	query result -> array of assoc. arrays
	 */
	function get ($what, $where="") {
		global $db;
		$this->result = array ();
		$RS = $db->exec ("SELECT $what FROM $this->name $where"); //, 1);
		if (!$RS) {
			$this->error = $db->report (msg ("select fail"));
			return false;
		}
		while ($row = $db->get_result_array ())
			array_push ($this->result, $row);
		return $this->result;
	}

	/**	Function executes a DB SELECT query as <code>get</code>, but returns just the first value.
	 *	@param	$what	string	The text between 'SELECT' and 'FROM'. Column names or "*".
	 *	@param	$where	string	Optional WHERE/ORDER clause.
	 *	@access	public
	 *	@return	query result -> one assoc. array
	 */
	function get_one ($what, $where="") {
		if ( !$this->get ("DISTINCTROW ".$what, $where) )
			return false;
		return $this->result[0];
	}
	
	/**	Return an associative array based on two DB entity columns.
	 *	@param	$ckey	string	First column name. Used as keys to associative array.
	 *	@param	$cval	string	Second column name. Used as values to associative array.
	 *	@param	$assoc	array	An array to put values to.
	 *	@param	$where	string	Optional WHERE/ORDER clause.
	 *	@access	public
	 *	@return	query result -> one assoc. array
	 */
	function get_assoc ($ckey, $cval, $assoc, $where="") {
		$what = "$ckey, $cval";
		if ( !$this->get ($what, $where) )
			return false;
		reset ($this->result);
		foreach ($this->result as $row)
			$assoc[$row[$ckey]] = $row[$cval];
		return $assoc;
	}
	/**	Function executing a DB DELETE query.
	 *	@param	$where	string	Compulsory WHERE clause.
	 *	@access	public
	 *	@return	number of deleted rows
	 */
	function delete ($where) {
		global $db;
		$this->result = array ();
		$RS = $db->exec ("DELETE FROM $this->name $where"); //, 1);
		if (!$RS) {
			$this->error = $db->report (msg ("delete fail"));
			return false;
		}
		return $db->get_affected_rows ();
	}

    /**	Assigns to local <code>error</code> variable.
     *	This is the ontrol point of whether use alert() or not.
	 *	@param $text string Text of the error message
     *	@return void
     *	@access public
     */
	function error ($text) {
		// alert type check (dialog with OK button)
		//$this->error = alert ($text);
		// message type check
		$this->error = $text;
	}

    /**	Returns text of the last error.
     *	@return string
     *	@access public
     */
	function get_error () {
		return $this->error;
	}
	
    /**	Prints formatted error message.
     *	Applications should call this function at a 'placeholder' that displays an error message if error occurs.
     *	@return void
     *	@access public
     */
	function error_report () {
		if (isset ($this->error)) {
			printf ("<br>%s", $this->error);
			return true;
		} else
			return false;
	}
	
	/**	A form select box renderer.<br>
	 *	Creates a select box based on the results of previous DB query.
	 *	@param	$name	Name of the select form item.
	 *	@param	$cval	Name of column or index to the query result, to be used as option's value="".
	 *	@param	$ctext	Name of column or index to the query result, to be used as option's label (text to display).
	 *	@param	$selected_value	
	 *	TODO: ..
	 *	@access	public
	 *	@return	void
	 */
	// it wound be possible to do this using get_assoc and there wound be no need to call ->get explicitly,
	// but like this, it's more common (* can be selected, not just two columns, and can be still worked with) ???
	function select_box ($name, $cval, $ctext, $selected_value, $begin_options="", $end_options="", $attributes="") {
		$sb_array = array ();
		if ($begin_options != "")
			$sb_array = $begin_options;
		reset ($this->result);
		foreach ($this->result as $row)
			$sb_array[$row[$cval]] = $row[$ctext];
		if ($end_options != "")
			$sb_array = array_merge ($sb_array, $end_options);
		render_select_box ($name, $sb_array, $selected_value, $attributes);
	}	// select_box()
}

?>