<?php

/*
 *	Article System
 *	Online journal management tool
 written in PHP/MySQL.
 *	This code is available at http://sourceforge.net/projects/artsys
 *
 *	Copyright (C) 2002 Jan Hnatek
 *	Distributed under the terms of the GNU General Public License
 *
 *	Date:	  07/05/2002
 *	Version:  0.1-1 (pre-alpha)      	
 */

###############################################################################
# form_process : form abstraction

// package artsys.forms

/**
 *	An abstract class implementing generic functionality for processing user's input.
 *	
 *	This class encapsulates generic functions for working with data coming from user forms. 
 *	Descendants must only override certain functions that perform context-specific tasks, 
 *	like custom checking of data, storing correct data, etc.<br>
 *	based on Yuri Makassiouk's (&lt;racer@bfpg.ru&gt;) FormProcessor
 *	
 *	@author	Jan Hnatek
 */
class form_process {
//----------------//

	// VARIABLES
	
	/**	Name of form
	 *	@var 	string
	 *	@acess	public
     */
    var $Name;
    
	/**	'True' if form is displayed for the first time.
     *	This value is set in the class constructor. 
	 *	If form finds its fields among values received from a POST, then it decides it was displayed before and submitted.
     *	@var 	bool
     *	@access private
     */
	var $FirstTime;
	
    /**	An associative array holding values of form's controls.
     *	@var	array
     *	@access private
     */
	var $Values;
	
    /**	An array that holds error messages
     *	This array should be filled by member functions that peform checking of user's input. 
	 *	It is used to make a descision weather form can be considered completed or not.
     *	@var 	array
     *	@access private
     *	@see check()
     *	@see custom_check()
     */
	var $Errors = array();
	
    /**	Error messages format.
     * You must set this value according to the appearance of error messages that would suite your application's needs
     *	@var 	string
     *	@access public
     *	@see 	error_report()
     */
	
	var $ErrorMessageFormat = '<br>%s';
	
    /**	Prefix that class uses to build names of HTML form elements.
     *	A string with only one special thing about it: it should be unique enough to not get 
	 *	names of controls that are managed by this form confused with names of other HTML elements.
     */
	var $fieldPrefix = 'form_data_';
	
	/**	Wizard object that this form is attached to.
	 *	Wizard is a sequence of form_process objects.
	 */
	var $Wizard;

    /**	Constructor of the class.
     *	Initializes variables and 'smells the air' deciding what state the form is in. 
	 *	If it can find values that correspond to this form among global variables,
	 *	it assumes that the form was submitted at least once, which means that values must be checked
	 *	and some further decisions made.
     *
     *	@param $Name string Name of the form
     *	@param $Wizard object A (reference to) wizard object this form is attached to
     *	@return void
     */
	function form_process ($Name, $Wizard=0) {
		$this->Name = $Name;
		$this->Wizard = $Wizard;
		//debug ("my wizard: ". $this->Wizard->name);
		
		$this->Values = $GLOBALS[$this->fieldPrefix.$this->Name];
		$this->FirstTime = (count($this->Values) == 0);
		
		if (!$this->FirstTime)
			$this->custom_check();
		
		if ($this->is_completed ()) {
			//debug ($this->Name." completed");
			$this->store_data();
			if ($this->Wizard != 0) {
				$this->Wizard->advance ();
				// TEMP
				//$this->display_form();
			}
		} else
			$this->display_form();
	}
    
    /**	Returns true if submitted data successfully passed verification.
     *	@return bool
     *	@access private
     */
	function is_completed () {
		return (!$this->FirstTime && count($this->Errors)<=0);
	}
    
    /**	Performs custom checking of data.
     *	This abstract function must be overridden in sub-classes to implement application logic.
	 *	Overridden functions must write error messages in Errors array, if a check fails.
     *	@return void
     *	@abstract
     *	@see $Errors
     */
	function custom_check () {
		//abstract
	}
    
    /**	Displays form's html.
     *	This abstract function must be overriden in sub-classes to implement actual form's displaying.
     *	@return void
     *	@abstract
     */
    function display_form () {
    	//abstract
	}
	
    /**	Stores user's input in the application.
     *	This abstract function must be overriden in sub-classes to implement application logic. 
	 *	It is called if form is considered completed in order to save user's input in a database or otherwise.
     *	@return void
     *	@abstract
     */
	function store_data () {
		//abstract
	}
	
    /**	Sets values of properties.
     *	Access method. Currently it is just a trivial assigning of a value to a local variable, 
	 *	but can be extended to do something useful.
     *
     *	@param $Name string Name of the object's field to be set
     *	@param $Value string Value to be assigned to the field
     *	@return void
     *	@access public
     */
	function set ($Name, $Value) {
		$this->$Name = $Value;
	}
	
    /**	Assigns to local Error[$Name] variable.
     *	It should be called from custom_check. This is the ontrol point of whether use alert() or not.
     *	@param $Name string Name of data element
	 *	@param $Text string Text of the error message
     *	@return void
     *	@access public
     */
	function error ($Name, $Text) {
		// alert type check (dialog with OK button)
		//$this->Errors[$Name] = alert ($Text);
		// message type check
		$this->Errors[$Name] = $Text;
	}
	
    /**	Prints formatted error message.
     *	Applications should call this function at a 'placeholder' that displays an error message if error occurs.
     *	@param $Name string Name of data element
     *	@return void
     *	@access public
     */
	function error_report ($Name) {
		if (isset ($this->Errors[$Name]))
			printf ($this->ErrorMessageFormat, $this->Errors[$Name]);
	}
	
	/** Returns initial value of a local variable
	 *	Returns false, if there is no such variable.
	 *	@param $Name string Name of data element
	 *	@return	unknown/false
	 *	@access	public
	 */
	function get_initial_value ($Name) {
		if (isset ($this->Values[$Name]))
			return $this->Values[$Name];
		else
			return false;
	}
	
	/** Echoes initial value of a local variable to output.
	 *	It's done by calling get_initial_value.
	 *	@param $Name string Name of data element
	 *	@return	void
	 *	@access	public
	 *	@see	get_initial_value()
	 */
	function initial_value ($Name) {
		echo $this->get_initial_value ($Name);
	}
	
	/**	Returns 'parent' wizard object (used for wizard sequence links etc.)
	 *	@return	wizard object
	 *	@access	public
	 */
	function get_wizard () {
		return $this->Wizard;
	}
}

?>
