<?php

/*
 *	Article System
 *	Online journal management tool
 written in PHP/MySQL.
 *	This code is available at http://sourceforge.net/projects/artsys
 *
 *	Copyright (C) 2002 Jan Hnatek
 *	Distributed under the terms of the GNU General Public License
 *
 *	Date:	  07/05/2002
 *	Version:  0.1-1 (pre-alpha)      	
 */

###############################################################################
# LIST_TABLE : A CLASS TO RENDER STRUCTURED DATA INTO TABLE
#

// DEBUG : uncomment this to get debug information
//define('LIST_TABLE_DEBUG', true);

 /*	HISTORY:
	1.0	/ 12.12.2001
		* "db_table" initial version, used only to render DB results
	1.1	/ 20.3.2002
		* created upper class object "list_table"
		* "db_table" made a subclass of "list_table" with custom constructor ($data structure is created there)
	1.2 / 1.4.2002
		* contructor now accepts the data as reference
 */

/**
 *	The <code>list_table</code> class was written to help rendering of data structured in rows/columns.
 *	<br>
 *	Features : 
 	<ul>
	<li>separately defined header</li>
	<li>periodically applied tr/td attributes (can be used to define switching background color, 
		or different field widths)</li>
	<li>fields as checkboxes</li>
	<li>"select all" button</li>
	<li>fields as (user defined) boolean values</li>
	<li>user defined indentation</li>
	</ul>
	<br>
	Examples of use:<br>
	<blockquote>
	<pre>
		// create new list_table object, give it input data
		// set indentation to 3 and enable "select all" button
		$table =& new list_table ($data, 3, true);
		
		// header column definition
		$table->use_header (array ("id", "forename", "name", "age", "is_a_member"));
		
		// define table and first row attributes
		$table->use_attributes ("border=\"1\"", "bgcolor=\"#C0C0C0\"", array ("width=\"20\"", "", "", "", ""));
		
		// enable color swapping : 	every 1st line "white"
		//							every 2nd line with bgcolor #e4e4e4
		// it's easily possible to add more colors
		$table->use_tr_attributes (array ("bgcolor=\"white\"", "bgcolor=\"#e4e4e4\""));
		
		// let "age" column (see header) will be centered
		$table->use_td_attributes (array ("", "", "", "align=center", ""));
		
		// convert first column of every line ("id"s, see header) into a checkbox
		$table->use_checkboxes (array (0), "A");
		
		// convert last column into a specially rendered boolea value
		// it can be done like this
		$table->use_boolean (array (3,4), "YES", "<b>NO</b>");
		// or let's use pictures to see if the person is_a_member
		$table->use_boolean (array (3,4), 
			"&lt;img src=\"img/yes.gif\" width=\"19\" height=\"8\" border=\"0\" alt=\"yes\"&gt;",
			"&lt;img src=\"img/no.gif\" width=\"19\" height=\"8\" border=\"0\" alt=\"no\"&gt;");
		
		// this was already set in constructor
		//$table->use_indent (3);
		//$table->use_check_all ();
		
		// and finally, render the whole table
		$table->render ();
		
		// we should also insert the script to handle "select all" button
		$table->insert_script ("item_list");
	</pre>
	</blockquote>
 *
 *	@version	1.2, 1.4.2002
 *	@author		Jan Hnatek
 */
class list_table {
//--------------//

	// VARIABLES
	
	// table data
	var $data;
	
	// attributes
	var $rows = 0;
	var $cols = 0;
	// render defaults
	var $indent = "";
		// main attributes
	var $table_attributes = "border=1 cellspacing=0 cellpadding=0";
	var $thead_attributes = "";
	var $theadtd_attributes;
		// repetitive attributes
	var $tr_attributes = "";
	var $td_attributes = "";
	
	// feature enable/disable
	var $header = false;
	var $header_cols;
	
	var $chboxes = false;
	var $chboxes_where = array ();
	var $chbox_prefix = "X";
	var $check_all = false;
	var $check_all_text = "select all";
	var $chname = "item_list";
	
	var $boolean = false;
	var $boolean_where;
	var $true_val = "1";
	var $false_val = "0";
	
	var $style;


	/**
	 *	Constructor for <code>list_table</code>.
	 *	@public
	 *	@param	&$data		reference to a 2D array of data to render
	 *	@param	$indent		indentation of the rendered table
	 *	@param	$chall		boolean to enable "select all" button
	 *	@param	$chname		name of the form (and script function)
	 *	@return	boolean
	 */
    function list_table (&$data, $indent="", $chall=false, $chname="item_list") {
		if (count ($data)) {
			$this->data = $data;
			$this->rows = count ($this->data);
			$this->cols = count ($this->data[0]);
			$this->_debug ("rows: ".$this->rows);
			$this->_debug ("columns: ".$this->cols);
		} else {
			$this->_error ("input data empty");
			return false;
		}
		$this->use_indent ($indent);
		$this->check_all = $chall;
		$this->chname = $chname;
	}


	// PRIVATE METHODS
		
		/**	Error reporting wrapper. Rewrite it to suit your needs.
		 *	@private
		 *	@param		$message	message to print.
		 *	@returns	void
		 */
		function _error ($message) {
			echo $message;
		}
		/**	Simple debug function for this class
		 *	@private
		 *	@param		$text	text to print.
		 *	@returns	void
		 */
		function _debug ($text) {
			if (defined('LIST_TABLE_DEBUG'))
				echo "\n<code style=\"color:gray\">$text</code><br>\n";
		}


	// PUBLIC METHODS
	
	/**	Use given table attributes on given table parts respectivelly
	 *	@param	$table		attributes of table tag
	 *	@param	$thead		attributes of the first tr, since I don't use thead
	 *	@param	$theadtd	array of td attributes to be repeatedly used within the first table row
	 *	@returns void
	 */
	function use_attributes ($table, $thead, $theadtd="") {
		$this->table_attributes = $table;
		$this->thead_attributes = $thead;
		if (strlen ($theadtd))
			$this->theadtd_attributes = $theadtd;
	}
	/**	Use given attributes on table rows
	 *	@param	$tr		array of tr attributes to be repeatedly used in the table rows
	 *	@returns void
	 */
	function use_tr_attributes ($tr) {
		$this->tr_attributes = $tr;
	}
	/**	Use given attributes on table columns
	 *	@param	$td		array of td attributes to be repeatedly used in the table columns
	 *	@returns void
	 */
	function use_td_attributes ($td) {
		$this->td_attributes = $td;
	}
	/**	Set table header data (first line of the table)
	 *	@param	$column_array	array of column names
	 *	@returns void
	 */
	function use_header ($column_array) {
		$this->header = true;
		$this->header_cols = $column_array;
	}
	
	/**	Render specified columns as checkboxes
	 *	@param	$column_numbers		array of column numbers
	 *	@param	$name_prefix		prefix string to use in name attribute of the input tag (name="prefix . data[i][j]")
	 *	@returns void
	 */
	function use_checkboxes ($column_numbers, $name_prefix) {
		$this->chboxes = true;
		$this->chboxes_where = $column_numbers;
		$this->chbox_prefix = $name_prefix;
	}
	/**	Enable "select all" button (checkbox) in the footer of the table
	 *	@param	$name	text to be printed next to button
	 *	@returns void
	 */
	function use_check_all ($name="select all") {
		$this->check_all = true;
		$this->check_all_text = $name;
	}
	/**	Render specified columns as (user defined) boolean values.<br>
	 *	@param	$column_numbers		array of column numbers
	 *	@param	$true_val			true value string (img tag)
	 *	@param	$false_val			false value string (img tag)
	 *	@returns void
	 */
	function use_boolean ($column_numbers, $true_val="1", $false_val="0") {
		$this->boolean = true;
		$this->boolean_where = $column_numbers;
		$this->true_val = $true_val;
		$this->false_val = $false_val;
	}
	/**	Set indentation of the whole rendered table.
	 *	@param	$num	number of indentation tabs
	 *	@returns void
	 */
	function use_indent ($num) {
		for ($i=0; $i < $num; $i++)
			$this->indent .= "	";
	}

	/**	Table rendering function : this creates the final table code.
	 *	@returns void
	 */
	function render () {
		$this->_table_begin ();
		$this->_table_head ();
		$this->_table_rows ();
		$this->_table_end ();
	}
	function _table_begin () {
		// create table start
		echo "\n".$this->indent."<table ". $this->table_attributes .">\n";
	}
	function _table_head () {
		// table head
		if ($this->header) {
			echo $this->indent."	<thead><tr ". $this->thead_attributes .">\n";
			for ($i=0; $i < $this->cols; $i++) {
				$col = $this->header_cols[$i];
				$theadtd_bonus = $this->theadtd_attributes[$i % count ($this->theadtd_attributes)];
				echo $this->indent."	<td $theadtd_bonus>". $col ."</td>\n";
			}
			echo $this->indent."</tr></thead>";
		} else echo $this->indent;
	}
	function _table_rows () {
		// create rows from data
		for ($i=0; $i < $this->rows; $i++) {
			$tr_bonus = $this->tr_attributes[$i % count ($this->tr_attributes)];
			// print row
			echo "	<tr $tr_bonus>\n";
			for ($j=0; $j < $this->cols; $j++) {
				$cell = $this->data[$i][$j];
				$td_bonus = $this->td_attributes[$j % count ($this->td_attributes)];
				if (($this->chboxes) && (in_array ($j, $this->chboxes_where))) {
					echo $this->indent."	<td $td_bonus>";
					?><input type="checkbox" value="<?= $cell?>" name="<?= $this->chbox_prefix.$cell?>"><?
					echo "</td>\n";
				} elseif (($this->boolean) && (in_array ($j, $this->boolean_where))) {
					if ($cell)
						$cell = $this->true_val;
					else
						$cell = $this->false_val;
					echo $this->indent."	<td $td_bonus>". $this->style . $cell ."</td>\n";
				} else
					echo $this->indent."	<td $td_bonus>".$this->style .  $cell ."</td>\n";
			}
			echo $this->indent."</tr>";
		}
	}
	function _table_end () {
		// optional check_all footer
		if ($this->check_all) {
			echo "	<tr ". $this->tr_attributes .">\n";
			echo $this->indent."	<td colspan=\"". $this->cols ."\"". $this->td_attributes .">";
			?><input type="checkbox" name="check_all" onclick="check_all_<?= $this->chname?> ()"><?= $this->check_all_text;
			echo "</td>\n";
			echo $this->indent."</tr>";
		}
		// finalize
		echo "</table>\n";
	}
	
	/**	Insert the script function for the table to handle the functionality of "select all" button.
	 *	@param	$name	name of the form, within which is the rendered table placed
	 *	@returns void
	 */
	function insert_script () {
		$this->_script_check_all ();
	}
	function _script_check_all () {	?>
	<script language="JavaScript1.1" type="text/javascript">
	<!--
	function check_all_<?= $this->chname?> () {
		var state = document.<?= $this->chname?>.check_all.checked;
		for (var i=0; i < document.<?= $this->chname?>.elements.length-1; i++)
			if (document.<?= $this->chname?>.elements[i].checked != state)
				document.<?= $this->chname?>.elements[i].click ();
	}
	-->
	</script>
<?	}

}	// class list_table

?>
